/*eslint no-use-before-define: 0*/
"use strict";

var architectureGraph = require("angular-architecture-graph");
var dot               = require("dot");

dot.templateSettings.strip = false;

module.exports = function (grunt) {

  var basePath = "node_modules/grunt-angular-architecture-graph/";
  // If this doesn't exist it means that we're executing this tasks tests
  // and the templates the base path is the root and not the
  // gruntfile.js location
  if (!grunt.file.exists("node_modules/grunt-angular-architecture-graph")) {
    basePath = "";
  }
  grunt.loadNpmTasks("grunt-graphviz");

  var files = {
    legend:  grunt.file.read(basePath + "templates/legend.def"),
    all:     grunt.file.read(basePath + "templates/all.def"),
    modules: grunt.file.read(basePath + "templates/modules.def"),
    module:  grunt.file.read(basePath + "templates/module.def")
  };

  var templates = {
    legendTemplate:  {},
    allTemplate:     {},
    modulesTemplate: {},
    moduleTemplate:  {}
  };

  // Available Helpers
  return {
    preprocessTemplates: preprocessTemplates,
    parseSrcFiles      : parseSrcFiles,
    analyseFiles       : analyseFiles,
    generateGraphFiles : generateGraphFiles,
    renderDotFiles     : renderDotFiles
  };

  /**
   * @param  {Object} options The task options passed in the gruntfile.
   * Preprocesses the graph template files. Replaces shape placeholders in the def files with either the default shapes or those
   * passed in the options, then loads the template into the templates object.
   */
  function preprocessTemplates (options) {
    // Replace placeholders.
    [
      "legend",
      "all",
      "modules",
      "module"
    ].forEach(function (file) {
      files[file] = files[file]
                      .replace(/\{1\}/g, options.shapeModules)
                      .replace(/\{2\}/g, options.shapeFactories)
                      .replace(/\{3\}/g, options.shapeDirectives)
                      .replace(/\{scheme\}/g, options.colorScheme);
    });

    // Prime the templates object.
    templates.legendTemplate  = dot.template(files.legend);
    templates.allTemplate     = dot.template(files.all);
    templates.modulesTemplate = dot.template(files.modules);
    templates.moduleTemplate  = dot.template(files.module);
  }

  /**
   * [parseSrcFiles description]
   * @param  {[type]} files [description]
   * @return {[type]}       [description]
   */
  function parseSrcFiles (files) {
    return files.src.filter(function(filepath) {
      // Warn on and remove invalid source files (if nonull was set).
      if (!grunt.file.exists(filepath)) {
        grunt.log.warn("Source file '" + filepath + "' not found.");
        return false;
      } else {
        return true;
      }
    }).map(function(filepath) {
      // Read file source.
      return {
        id: filepath,
        text: grunt.file.read(filepath)
      };
    });
  }

  function analyseFiles (files, options) {
    var graph = architectureGraph(files, options);
    return graph.angular;
  }

  /**
   * [generateGraphFiles description]
   * @param  {[type]} result [description]
   * @return {[type]}        [description]
   */
  function generateGraphFiles (angular, files) {
    generateLegendGraph(files);
    generateAllGraph(angular, files);
    generateModulesGraph(angular, files);
    angular.modules.forEach(function (module) {
      generateModuleGraph(module, files);
    });
    return angular;
  }

  function renderDotFiles (files) {
    var dest = files.dest;
    var config = {
      graphviz: {
        target: {
          files: {}
        }
      }
    };

    // Configure png files for all generated .dot files
    var diagrams = grunt.file.expand(dest + "/dot/**/*.dot");
    diagrams.forEach(function (diagram) {
      // we just want the diagram name and folder so we remove
      // the remainder of the path
      diagram = diagram.replace(dest + "/dot", "");
      diagram = diagram.replace(".dot", "");

      // finally, add this diagram's config
      // to the graphviz tasks config object
      config.graphviz.target.files[dest + "/png" + diagram + ".png"] = dest + "/dot" + diagram + ".dot";
    });

    // Add configuration and run graphviz task
    grunt.config.merge(config);
    grunt.task.run(["graphviz"]);
  }

  /*-------------------
   * Private
   *-------------------
   */

  function generateLegendGraph (files) {
    var legendResult = templates.legendTemplate();

    grunt.file.write(files.dest + "/dot/legend.dot", legendResult);
  }

  function generateAllGraph (angular, files) {
    var allResult = templates.allTemplate({
      modules: angular.modules
    });

    grunt.file.write(files.dest + "/dot/all.dot", allResult);
  }

  function generateModulesGraph (angular, files) {
    var modulesResult = templates.modulesTemplate({
        modules: angular.modules
    });
    grunt.file.write(files.dest + "/dot/modules.dot", modulesResult);
  }

  function generateModuleGraph (module, files) {
    var moduleResult = templates.moduleTemplate(module);
    grunt.file.write(files.dest + "/dot/modules/" + module.name + ".dot", moduleResult);
  }


};
